
[toc]

## 项目介绍

> node 8.11.0+ (推荐12.18.3)
> npm 6.14.6

基于vue-cli3完善的脚手架项目，增加一些基础通用配置。
包含配置如下：

- [x] 环境区分：分为开发、测试和生产环境
- [x] 通过`require.context`自动化vue-router配置
- [x] 配置`layout`布局组件
- [x] 按需引入lodash
- [x] moment二次封装进行汉化，并且配置为过滤器、实例方式调用
- [x] 生产环境打包去掉console
- [x] 模块化vuex，并通过`require.context`自动化配置
- [x] 配置`global`全局组件
- [x] 配置`common`公共组 件
- [x] 配置全局自定义过滤器
- [x] 配置全局自定义指令
- [x] 配置`utils`方法并通过实例调用
- [x] 配置错误页面
- [x] `eslint` + `prettier`规范代码风格、格式化及语法校验
- [x] `husky` + `lint-staged`拦截commit，提交时做代码风格校验及修复
- [x] `commitizen` + `cz-conventional-changlog`提供规范统一的的commiteMessage模板
- [x] `husky` + `commitlint`在commit时做commitMessage规范校验
- [x] `standard-version`自动化发版流程及生成规范化的changelog
- [x] Symbol的在线链接方式引入iconfont
- [x] 新增axios配置：全局拦截器；防止重复请求；路由切换取消当前页面所有pending（可指定）请求；实例中取消请求等
- [x] api统一管理


## 运行
安装依赖：
```
npm install
```

本地开发：
```
npm run serve
```

测试环境打包：
```
npm run passports
```

生产环境打包：
```
npm run build
```

打包后本地预览：
```
npm run start
```

格式校验及自动修复：
修复eslint报错：
```
npm run fix
```

修复prettier报错
```
npm run prettier
```

## 开发流程及规范

### 分支说明

### 开发流程

### 代码规范

1. 任何人本地出现代码格式相关校验的出错，不得强制忽略或者提交，必须在本地处理完再提交。

### 发版流程
1. 切换到master分支并且执行`git pull`更新代码
2. 将测试完成要发版的分支合并到master分支
3. 自动更新`package.json`中的版本号，更新`changelog`,同时打版到master分支：

    第一个版本，需要根据当前package.json中的版本初始化tag及changelog：`yarn release:init`;<br>
    更新修订版本号：`yarn release:patch`<br>
    更新次版本号：`yarn release:minor`<br>
    更新主版本号：`yarn release:major`<br>

4. 推送当前分支上的版本号到远程仓库：`git push origin --tags`或者`git push --flollow-tags`


> 版本号一般为3位：x.x.x,每一位对应的含义分别是major主版本号、monor次版本号、patch修订版本号。


## 相关指南

### 目录结构

```
├── CHANGELOG.md  // 更新日志
├── README.md
├── babel.config.js
├── config   // 配置文件目录
│   └── baseUrl.js
├── package.json
├── public   // 静态资源目录（不会被webpack打包）
│   ├── favicon.ico
│   ├── img
│   │   └── icons
│   ├── index.html
│   └── robots.txt
├── src
│   ├── App.vue
│   ├── api     // api管理、配置
│   │   └── index.js
│   ├── assets  // 静态资源目录（样式、图片等），会被webpack打包
│   │   ├── logo.png
│   │   └── style
│   ├── components  // 组件目录
│   │   ├── global  // 全局组件
│   │   ├── common  // 公共组件
│   │   ├── header
│   │   ├── layout
│   │   ├── rightPanel
│   │   └── sideNav
│   ├── directives   // 自定义指令
│   │   ├── follower.js
│   │   ├── index.js   // 指令配置导出文件
│   │   └── placeholder.js
│   ├── filters   // 自定义过滤器
│   │   └── index.js   // 过滤器导出文件
│   ├── main.js
│   ├── plugins   // 第三方库二次封装
│   │   └── moment.js
│   ├── registerServiceWorker.js
│   ├── router
│   │   ├── home.router.js  // 具体路由文件
│   │   ├── index.js   // 路由配置及导出文件
│   │   ├── page1.router.js
│   │   └── page2.router.js
│   ├── store
│   │   ├── global.js   // 全局vuex
│   │   ├── index.js  // vuex配置及导出页面
│   │   └── modules  // 模块化vuex
│   ├── utils
│   │   └── index.js
│   └── views  // 页面/路由组件
│       ├── 404.vue
│       ├── home.vue
│       ├── page1.vue
│       └── page2.vue
├── vue.config.js
└── yarn.lock
```

注意：
1. 页面组件（通俗理解为与路由对应的组件，也可称路由组件）统一放在`views`目录下。
2. 页面组件内的小组件、功能组件统一放在`components`目录下。例如，页面组件`pageA.vue`中拆分出了`pageheader.vue`组件，那么在`components`目录下新建一个**与页面同名的`pageA`目录**，`pageA`的相关组件放进去即可。
3. 注意区分`global`目录与`common`目录的不同：
    `global`：全局组件，可能需要在多处引入使用<br>
    `common`：公共组件，全局生效，但在app.vue中引入即可，无需多次引入使用<br>


### vuex配置及使用
全局类的数据放在`/src/store/global.js`中。<br>
模块化的数据放在`/src/store/modules`目录中，新建与模块对应的js文件即可，会自动引入并注册。注意，模块化js文件中`namespaced`属性必填。

### vue-router配置及使用
如果新增路由页面`xxx.vue`，在`src/views`目录下新增`xxx.vue`页面之后，在`src/router`目录下新增对应的路由配置文件`xxx.router.js`并将配置通过`export default`导出，配置方式与原有配置无异。导出后即可，会自动注册路由。

### layout布局组件配置及使用
需要新增布局时，在`src/components/layout`目录下新增`.vue`文件，类似`App.vue`,其中`<router-view></router-view>`用于渲染路由组件。<br>
文件内的`name`属性为布局名称，需要应用此布局的页面，在对应的路由配置文件`xxx.router.js`中，配置`meta`属性，`meta.layout`赋值为布局名称即可。


### 全局组件配置及使用
在`/src/components/global`目录下添加`.vue`组件，并声明`name`属性即可，会自动引入并挂载到vue实例上。<br>
注意，`name`属性必填，且`name`的值就是使用时的组件名称。<br>

### axios使用说明
TODO

## CHANGELOG

[查看CHANGELOG](./CHANGELOG.md)


## Customize configuration
See [Configuration Reference](https://cli.vuejs.org/config/).
****
