import Vue from 'vue'
import App from './App.vue'
import router from './router'
import store from './store'
import './registerServiceWorker'
import './assets/style/index.scss'
import * as requests from '@/plugins/axios/index'
import api from '@/api/index.js'
import VueBus from 'vue-bus'
import VModal from 'vue-js-modal'
import moment from '@/plugins/moment.js'
import layoutComponents from '@/components/layout/index.js'
import globalComponents from '@/components/global/index.js'
import filters from '@/filters/index.js'
import directives from '@/directives/index.js'
import utils from '@/utils/index.js'

// 全局注册layout组件
Vue.use(layoutComponents)
// 注册自定义全局组件
Object.keys(globalComponents).forEach((compName) => Vue.component(compName, globalComponents[compName]))
// 注 册全局自定义filter
Object.keys(filters).forEach((key) => Vue.filter(key, filters[key]))
// 注册全局自定义directive
Object.keys(directives).forEach((key) => Vue.directive(key, directives[key]))

Vue.use(VueBus)
Vue.use(VModal)

Vue.config.productionTip = false
Vue.prototype.$request = requests.request
Vue.prototype.$intactRequest = requests.intactRequest
Vue.prototype.$api = api
Vue.prototype.$mt = moment
Vue.prototype.$ = utils
new Vue({
    router,
    store,
    render: (h) => h(App)
}).$mount('#app')
