import Vue from 'vue'
import VueRouter from 'vue-router'
import store from '@/store/index.js'
import { request } from '@/plugins/axios/index'
Vue.use(VueRouter)

let routes = []

// 通配符404页面
const wildcardRoutes = {
    path: '*',
    name: '*',
    // hidden: true,
    meta: {
        layout: 'error',
        title: '错误页面'
    },
    component: () => import(/* webpackChunkName: "404" */ '@/views/404.vue')
}

// 引入所有路由文件
function importAll(r) {
    r.keys().forEach((path) => routes.push(r(path).default || r(path)))
}
importAll(require.context('@/router/', true, /\.router\.js/))

routes.push(wildcardRoutes)

const router = new VueRouter({
    mode: 'history',
    base: process.env.BASE_URL,
    routes,
    scrollBehavior(to, from, savedPosition) {
        // if (savedPosition) {
        //     return savedPosition
        // } else {
        //     if (from.meta.keepAlive) {
        //         from.meta.savedPosition = document.body.scrollTop
        //     }
        //     return { x: 0, y: to.meta.savedPosition || 0 }
        // }
        return new Promise((resolve, reject) => {
            setTimeout(() => {
                if (savedPosition) {
                    resolve(savedPosition)
                } else {
                    if (from.meta.keepAlive) {
                        from.meta.savedPosition = document.body.scrollTop
                    }
                    resolve({ x: 0, y: to.meta.savedPosition || 0 })
                }
            }, 500)
        })
    }
})

// 路由全局前置守卫
router.beforeEach((to, from, next) => {
    // 路由变化时取消当前所有非全局的pending状态的请求
    request.clearPendingPool()
    // 未登录情况下访问的页面需要登录，自动跳转到指定页面（登录页）
    if (to.meta.needAuth && !store.getters['authed']) next({ name: 'page2' })
    else next()
})

export default router
